@@ -6,7 +6,6 @@ module Fog
     class Compute < Fog::Service
       requires   :libvirt_uri
       recognizes :libvirt_username, :libvirt_password
-      recognizes :libvirt_ip_command
 
       model_path 'fog/libvirt/models/compute'
       model       :server
@@ -50,37 +49,13 @@ module Fog
 
       module Shared
         include Fog::Libvirt::Util
-      end
 
-      class Mock
-        include Shared
-        def initialize(options={})
-          # libvirt is part of the gem => ruby-libvirt
-          require 'libvirt'
-        end
-
-        private
-
-        def client
-          return @client if defined?(@client)
-        end
-
-        #read mocks xml
-        def read_xml(file_name)
-          file_path = File.join(File.dirname(__FILE__),"requests","compute","mock_files",file_name)
-          File.read(file_path)
-        end
-      end
-
-      class Real
-        include Shared
         attr_reader :client
         attr_reader :uri
-        attr_reader :ip_command
 
         def initialize(options={})
+          super()
           @uri = ::Fog::Libvirt::Util::URI.new(enhance_uri(options[:libvirt_uri]))
-          @ip_command = options[:libvirt_ip_command]
 
           # libvirt is part of the gem => ruby-libvirt
           begin
@@ -135,6 +110,20 @@ module Fog
           uri+append
         end
       end
+
+      class Real
+        include Shared
+      end
+
+      class Mock
+        include Shared
+
+        def enhance_uri(uri)
+          uri = 'test:///default' unless ::URI.parse(uri).scheme == 'test'
+
+          super(uri)
+        end
+      end
     end
   end
 end
