@@ -2,10 +2,10 @@ require 'fog/core/collection'
 require 'fog/libvirt/models/compute/node'
 
 module Fog
-  module Compute
-    class Libvirt
+  module Libvirt
+    class Compute
       class Nodes < Fog::Collection
-        model Fog::Compute::Libvirt::Node
+        model Fog::Libvirt::Compute::Node
 
         def all(filter={ })
           load(service.get_node_info)
