@@ -13,6 +13,10 @@ module Fog
 
         attribute :cpus
         attribute :cputime
+        attribute :firmware
+        attribute :firmware_features
+        attribute :secure_boot
+        attribute :loader_attributes
         attribute :os_type
         attribute :memory_size
         attribute :max_memory_size
@@ -30,6 +34,7 @@ module Fog
         attribute :cpu
         attribute :hugepages
         attribute :guest_agent
+        attribute :video
         attribute :virtio_rng
 
         attribute :state
@@ -83,19 +88,24 @@ module Fog
         end
 
         def mac
-          nics.first.mac if nics && nics.first
+          nics&.first&.mac
         end
 
         def disk_path
           volumes.first.path if volumes and volumes.first
         end
 
-        def destroy(options={ :destroy_volumes => false, :flags => 0 })
+        def destroy(options={ :destroy_volumes => false, :flags => ::Libvirt::Domain::UNDEFINE_NVRAM })
           poweroff unless stopped?
-          if options.fetch(:flags, 0).zero?
+          flags = options.fetch(:flags, ::Libvirt::Domain::UNDEFINE_NVRAM)
+          if flags.zero?
             service.vm_action(uuid, :undefine)
           else
-            service.vm_action(uuid, :undefine, options[:flags])
+            # the test driver doesn't support UNDEFINE_NVRAM
+            if service.uri.driver == 'test'
+              flags ^= ::Libvirt::Domain::UNDEFINE_NVRAM
+            end
+            service.vm_action(uuid, :undefine, flags)
           end
           volumes.each { |vol| vol.destroy } if options[:destroy_volumes]
           true
@@ -281,14 +291,31 @@ module Fog
               end
 
               xml.vcpu(cpus)
-              xml.os do
+              os_tags = {}
+
+              os_tags[:firmware] = firmware if firmware == 'efi'
+
+              xml.os(**os_tags) do
                 type = xml.type(os_type, :arch => arch)
                 type[:machine] = "q35" if ["i686", "x86_64"].include?(arch)
 
                 boot_order.each do |dev|
                   xml.boot(:dev => dev)
                 end
+
+                loader_attributes&.each do |key, value|
+                  xml.loader(key => value)
+                end
+
+                if firmware == "efi" && firmware_features&.any?
+                  xml.firmware do
+                    firmware_features.each_pair do |key, value|
+                      xml.feature(:name => key, :enabled => value)
+                    end
+                  end
+                end
               end
+
               xml.features do
                 xml.acpi
                 xml.apic
@@ -301,7 +328,7 @@ module Fog
                   end
                 else
                   xml.cpu(
-                    :mode => cpu.dig(:model, :name) || "host-passthrough",
+                    :mode => cpu.fetch(:mode, "host-passthrough"),
                     :check => cpu.fetch(:check, "none"),
                     :migratable => cpu.fetch(:migratable, "on")
                   )
@@ -336,7 +363,7 @@ module Fog
                         end
                       end
 
-                      xml.target(:dev => target_device, :bus => args["bus_type"] == "virtio" ? "virtio" : "scsi")
+                      xml.target(:dev => target_device, :bus => ceph_args["bus_type"] == "virtio" ? "virtio" : "scsi")
                     end
                   else
                     is_block = volume.path.start_with?("/dev/")
@@ -356,14 +383,15 @@ module Fog
                   xml.disk(:type => "file", :device => "cdrom") do
                     xml.driver(:name => "qemu", :type => "raw")
                     xml.source(:file => "#{iso_dir}/#{iso_file}")
-                    xml.target(:dev => "hdc", :bus => "ide")
+                    xml.target(:dev => "sda", :bus => "scsi")
                     xml.readonly
-                    xml.address(:type => "drive", :controller => 0, :bus => 1, :unit => 0)
+                    xml.address(:type => "drive", :controller => 0, :bus => 0, :unit => 0)
                   end
                 end
 
                 nics.each do |nic|
                   xml.interface(:type => nic.type) do
+                    xml.mac(:address => nic.mac) if nic.mac
                     if nic.type == "bridge"
                       xml.source(:bridge => nic.bridge)
                     else
@@ -408,10 +436,10 @@ module Fog
                     graphics.autoport = "yes"
                   end
                   graphics.listen = display[:listen] unless display[:listen].empty?
-                  graphics.password = display[:password] unless display[:password].empty?
+                  graphics.passwd = display[:password] if display[:password] && !display[:password].empty?
 
                   xml.video do
-                    xml.model(:type => "cirrus", :vram => 9216, :heads => 1)
+                    xml.model(video)
                   end
                 end
               end
@@ -440,160 +468,26 @@ module Fog
           args
         end
 
-        # This tests the library version before redefining the address
-        # method for this instance to use a method compatible with
-        # earlier libvirt libraries, or uses the dhcp method from more
-        # recent releases.
-        def addresses(service_arg=service, options={})
-          addresses_method = self.method(:addresses_dhcp)
-          # check if ruby-libvirt was compiled against a new enough version
-          # that can use dhcp_leases, as otherwise it will not provide the
-          # method dhcp_leases on any of the network objects.
-          has_dhcp_leases = true
-          begin
-            service.networks.first.dhcp_leases(self.mac)
-          rescue NoMethodError
-            has_dhcp_leases = false
-          rescue
-            # assume some other odd exception.
-          end
-
-          # if ruby-libvirt not compiled with support, or remote library is
-          # too old (must be newer than 1.2.8), then use old fallback
-          if not has_dhcp_leases or service.libversion() < 1002008
-            addresses_method = self.method(:addresses_ip_command)
-          end
-
-          # replace current definition for this instance with correct one for
-          # detected libvirt to perform check once for connection
-          (class << self; self; end).class_eval do
-            define_method(:addresses, addresses_method)
-          end
-          addresses(service_arg, options)
-        end
-
-        def ssh_ip_command(ip_command, uri)
-          # Retrieve the parts we need from the service to setup our ssh options
-          user=uri.user #could be nil
-          host=uri.host
-          keyfile=uri.keyfile
-          port=uri.port
-
-          # Setup the options
-          ssh_options={}
-          ssh_options[:keys]=[ keyfile ] unless keyfile.nil?
-          ssh_options[:port]=port unless keyfile.nil?
-          ssh_options[:paranoid]=true if uri.no_verify?
-
-          begin
-            result=Fog::SSH.new(host, user, ssh_options).run(ip_command)
-          rescue Errno::ECONNREFUSED
-            raise Fog::Errors::Error.new("Connection was refused to host #{host} to retrieve the ip_address for #{mac}")
-          rescue Net::SSH::AuthenticationFailed
-            raise Fog::Errors::Error.new("Error authenticating over ssh to host #{host} and user #{user}")
-          end
-
-          # Check for a clean exit code
-          if result.first.status == 0
-            return result.first.stdout.strip
-          else
-            # We got a failure executing the command
-            raise Fog::Errors::Error.new("The command #{ip_command} failed to execute with a clean exit code")
-          end
-        end
-
-        def local_ip_command(ip_command)
-          # Execute the ip_command locally
-          # Initialize empty ip_address string
-          ip_address=""
-
-          IO.popen("#{ip_command}") do |p|
-            p.each_line do |l|
-              ip_address+=l
-            end
-            status=Process.waitpid2(p.pid)[1].exitstatus
-            if status!=0
-              raise Fog::Errors::Error.new("The command #{ip_command} failed to execute with a clean exit code")
-            end
-          end
-
-          #Strip any new lines from the string
-          ip_address.chomp
-        end
-
-        # Locale-friendly removal of non-alpha nums
-        DOMAIN_CLEANUP_REGEXP = Regexp.compile('[\W_-]')
-
-        # This retrieves the ip address of the mac address using ip_command
-        # It returns an array of public and private ip addresses
-        # Currently only one ip address is returned, but in the future this could be multiple
-        # if the server has multiple network interface
-        def addresses_ip_command(service_arg=service, options={})
-          mac=self.mac
-
-          # Aug 24 17:34:41 juno arpwatch: new station 10.247.4.137 52:54:00:88:5a:0a eth0.4
-          # Aug 24 17:37:19 juno arpwatch: changed ethernet address 10.247.4.137 52:54:00:27:33:00 (52:54:00:88:5a:0a) eth0.4
-          # Check if another ip_command string was provided
-          ip_command_global=service_arg.ip_command.nil? ? 'grep $mac /var/log/arpwatch.log|sed -e "s/new station//"|sed -e "s/changed ethernet address//g" |sed -e "s/reused old ethernet //" |tail -1 |cut -d ":" -f 4-| cut -d " " -f 3' : service_arg.ip_command
-          ip_command_local=options[:ip_command].nil? ? ip_command_global : options[:ip_command]
-
-          ip_command="mac=#{mac}; server_name=#{name.gsub(DOMAIN_CLEANUP_REGEXP, '_')}; "+ip_command_local
-
-          ip_address=nil
-
-          if service_arg.uri.ssh_enabled?
-            ip_address=ssh_ip_command(ip_command, service_arg.uri)
-          else
-            # It's not ssh enabled, so we assume it is
-            if service_arg.uri.transport=="tls"
-              raise Fog::Errors::Error.new("TlS remote transport is not currently supported, only ssh")
-            end
-            ip_address=local_ip_command(ip_command)
-          end
-
-          # The Ip-address command has been run either local or remote now
-
-          if ip_address==""
-            #The grep didn't find an ip address result"
-            ip_address=nil
-          else
-            # To be sure that the command didn't return another random string
-            # We check if the result is an actual ip-address
-            # otherwise we return nil
-            unless ip_address=~/^(\d{1,3}\.){3}\d{1,3}$/
-              raise Fog::Errors::Error.new(
-                        "The result of #{ip_command} does not have valid ip-address format\n"+
-                            "Result was: #{ip_address}\n"
-                    )
-            end
-          end
-
-          return { :public => [ip_address], :private => [ip_address]}
-        end
-
         # This retrieves the ip address of the mac address using dhcp_leases
         # It returns an array of public and private ip addresses
         # Currently only one ip address is returned, but in the future this could be multiple
         # if the server has multiple network interface
-        def addresses_dhcp(service_arg=service, options={})
-          mac=self.mac
-
+        def addresses(service_arg=service, options={})
           ip_address = nil
-          nic = self.nics.find {|nic| nic.mac==mac}
-          if !nic.nil?
+          if (nic = self.nics&.first)
             net = service.networks.all(:name => nic.network).first
-            if !net.nil?
-              leases = net.dhcp_leases(mac, 0)
-              # Assume the lease expiring last is the current IP address
-              ip_address = leases.sort_by { |lse| lse["expirytime"] }.last["ipaddr"] if !leases.empty?
-            end
+            # Assume the lease expiring last is the current IP address
+            ip_address = net&.dhcp_leases(nic.mac)&.max_by { |lse| lse["expirytime"] }&.dig("ipaddr")
           end
 
           return { :public => [ip_address], :private => [ip_address] }
         end
 
+        # Locale-friendly removal of non-alpha nums
+        DOMAIN_CLEANUP_REGEXP = Regexp.compile('[\W_-]')
+
         def ip_address(key)
-          addresses[key].nil? ? nil : addresses[key].first
+          addresses[key]&.first
         end
 
         def initialize_nics
@@ -664,7 +558,9 @@ module Fog
             :cpu                    => {},
             :hugepages              => false,
             :guest_agent            => true,
+            :video                  => {:type => "cirrus", :vram => 9216, :heads => 1},
             :virtio_rng             => {},
+            :firmware_features      => { "secure-boot" => "no" },
           }
         end
 
@@ -678,7 +574,7 @@ module Fog
         end
 
         def default_display
-          {:port => '-1', :listen => '127.0.0.1', :type => 'vnc', :password => '' }
+          {:port => '-1', :listen => '127.0.0.1', :type => 'vnc' }
         end
       end
     end
