@@ -39,7 +39,8 @@ module Fog
 
           raise Fog::Errors::Error.new('Reserving an existing volume may create a duplicate') if key
           @xml ||= to_xml
-          self.path = service.create_volume(pool_name, xml).path
+          self.id = service.create_volume(pool_name, xml).key
+          reload
         end
 
         # Destroy a volume
@@ -68,8 +69,8 @@ module Fog
           new_volume      = self.dup
           new_volume.key  = nil
           new_volume.name = new_name
-          new_volume.path = service.clone_volume(pool_name, new_volume.to_xml, self.name).path
 
+          new_volume.id = service.clone_volume(pool_name, new_volume.to_xml, self.name).key
           new_volume.reload
         end
 
@@ -78,8 +79,46 @@ module Fog
           service.upload_volume(pool_name, name, file_path)
         end
 
+        def to_xml
+          builder = Nokogiri::XML::Builder.new do |xml|
+            xml.volume do
+              xml.name(name)
+
+              allocation_size, allocation_unit = split_size_unit(allocation)
+              xml.allocation(allocation_size, :unit => allocation_unit)
+
+              capacity_size, capacity_unit = split_size_unit(capacity)
+              xml.capacity(capacity_size, :unit => capacity_unit)
+
+              xml.target do
+                xml.format(:type => format_type)
+                xml_permissions(xml)
+              end
+
+              if backing_volume
+                xml.backingStore do
+                  xml.path(backing_volume.path)
+                  xml.format(:type => backing_volume.format_type)
+                  xml_permissions(xml)
+                end
+              end
+            end
+          end
+
+          builder.to_xml
+        end
+
         private
 
+        def xml_permissions(xml)
+          xml.permissions do
+            xml.owner(owner) if owner
+            xml.group(group) if group
+            xml.mode('0744')
+            xml.label('virt_image_t')
+          end
+        end
+
         def image_suffix
           return "img" if format_type == "raw"
           format_type
