@@ -1,16 +1,18 @@
 module Fog
   module Libvirt
     class Compute
-      class Real
+      module Shared
         def define_domain(xml)
           client.define_domain_xml(xml)
         end
       end
 
+      class Real
+        include Shared
+      end
+
       class Mock
-        def define_domain(xml)
-          ::Libvirt::Domain.new()
-        end
+        include Shared
       end
     end
   end
