@@ -1,15 +1,18 @@
 module Fog
   module Libvirt
     class Compute
-      class Real
+      module Shared
         def define_pool(xml)
           client.define_storage_pool_xml(xml)
         end
       end
 
+      class Real
+        include Shared
+      end
+
       class Mock
-        def define_pool(xml)
-        end
+        include Shared
       end
     end
   end
