@@ -1,17 +1,19 @@
 module Fog
   module Libvirt
     class Compute
-      class Real
+      module Shared
         #shutdown the interface
         def destroy_interface(uuid)
           client.lookup_interface_by_uuid(uuid).destroy
         end
       end
 
+      class Real
+        include Shared
+      end
+
       class Mock
-        def destroy_interface(uuid)
-          true
-        end
+        include Shared
       end
     end
   end
