@@ -1,16 +1,18 @@
 module Fog
   module Libvirt
     class Compute
-      class Real
+      module Shared
         def destroy_network(uuid)
           client.lookup_network_by_uuid(uuid).destroy
         end
       end
 
+      class Real
+        include Shared
+      end
+
       class Mock
-        def destroy_network(uuid)
-          true
-        end
+        include Shared
       end
     end
   end
