@@ -1,7 +1,7 @@
 module Fog
   module Libvirt
     class Compute
-      class Real
+      module Shared
         def vm_action(uuid, action, *params)
           domain = client.lookup_domain_by_uuid(uuid)
           domain.send(action, *params)
@@ -9,10 +9,12 @@ module Fog
         end
       end
 
+      class Real
+        include Shared
+      end
+
       class Mock
-        def vm_action(uuid, action, *params)
-          true
-        end
+        include Shared
       end
     end
   end
