@@ -1,7 +1,7 @@
 Shindo.tests('Fog::Compute[:libvirt] | network model', ['libvirt']) do
 
   networks = Fog::Compute[:libvirt].networks
-  network = networks.last
+  network = networks.first
 
   tests('The network model should') do
     tests('have the action') do
@@ -9,7 +9,7 @@ Shindo.tests('Fog::Compute[:libvirt] | network model', ['libvirt']) do
       test('dhcp_leases') { network.respond_to? 'dhcp_leases' }
     end
     tests('have a dhcp_leases action that') do
-      test('returns an array') { network.dhcp_leases('99:88:77:66:55:44', 0).kind_of? Array }
+      test('returns an array') { network.dhcp_leases('aa:bb:cc:dd:ee:ff', 0).kind_of? Array }
     end
     tests('have attributes') do
       model_attribute_hash = network.attributes
@@ -28,4 +28,16 @@ Shindo.tests('Fog::Compute[:libvirt] | network model', ['libvirt']) do
     test('be a kind of Fog::Libvirt::Compute::Network') { network.kind_of? Fog::Libvirt::Compute::Network }
   end
 
+  tests("to_xml") do
+    test("default") do
+      expected = <<~NETWORK
+        <?xml version="1.0"?>
+        <network>
+          <name>default</name>
+          <bridge name="virbr0" stp="on" delay="0"/>
+        </network>
+      NETWORK
+      network.to_xml == expected
+    end
+  end
 end
