@@ -1,6 +1,6 @@
 Shindo.tests('Fog::Compute[:libvirt] | volume model', ['libvirt']) do
 
-  volume = Fog::Compute[:libvirt].servers.all.select{|v| v.name !~ /^fog/}.first.volumes.first
+  volume = Fog::Compute[:libvirt].volumes.create(:name => 'fog_test')
 
   tests('The volume model should') do
     tests('have attributes') do
@@ -35,4 +35,24 @@ Shindo.tests('Fog::Compute[:libvirt] | volume model', ['libvirt']) do
     test('succeed') { volume.xml == new_vol.xml }
   end
 
+  test('to_xml') do
+    test('default') do
+      expected = <<~VOLUME
+        <?xml version="1.0"?>
+        <volume>
+          <name>fog_test</name>
+          <allocation unit="G">1</allocation>
+          <capacity unit="G">10</capacity>
+          <target>
+            <format type="raw"/>
+            <permissions>
+              <mode>0744</mode>
+              <label>virt_image_t</label>
+            </permissions>
+          </target>
+        </volume>
+      VOLUME
+      volume.to_xml == expected
+    end
+  end
 end
